#!/usr/bin/perl
use strict;
use File::Basename;
use Cwd 'abs_path';


my %cmd_args = (
	user 	=> undef,
	passwd 	=> undef,
	filespace => undef,
	nodename => undef,
	filename => undef,
	filelist => undef, 
	); 

sub check_cmd_args
{
	my ($cmd_args) = @_;

    foreach my $cmd_param (@ARGV)
    {
        if ($cmd_param =~ m/^--(passwd)=(.+)/) 
        {
        	$cmd_args{$1} = $2;
        }
        elsif ($cmd_param =~ m/^--(user)=(.+)/)
        {
        	$cmd_args{$1} = $2;
        }
        elsif ($cmd_param =~ m/^--(filespace)=(.+)/)
        {
        	$cmd_args{$1} = $2;
        }
        elsif ($cmd_param =~ m/^--(nodename)=(.+)/)
        {
        	$cmd_args{$1} = $2;
        }
        elsif ($cmd_param =~ m/^--(filename)=(.+)/)
        {
        	$cmd_args{$1} = $2;
        }
        elsif ($cmd_param =~ m/^--(filelist)=(.+)/)
        {
        	$cmd_args{$1} = $2;
        }
        else
        {
        	print "\nInvalid parameter: $cmd_param\n";
        	print_help();
        	exit();
        }
    }

    my $missing_req_argument = 0;
	for my $key (keys %cmd_args)
	{		
		if (!$cmd_args{$key})
		{
			print "--$key argument is required.\n";
			$missing_req_argument = 1;
		}		
	}

	if ($missing_req_argument)
	{
		print_help();
		exit;
	}
}

sub print_help
{
    my $script_name = basename($0);

    print "\nUsage:\n";
    print "$script_name --user=<TSM-Server user>
               --passwd=<TSM Server password> 
               --filespace=<TSM filespace> 
               --nodename=<nodename> 
               --filename=<filename>
               --filelist=<filelist path>\n\n";

}

sub create_file_list
{
	my ($cmd_args) = @_;

	my $command = "dsmadmc -id=$cmd_args{user} -pa=$cmd_args{passwd} -DISPL=LIST \"SELECT s.NODE_NAME, s.FILESPACE_NAME, s.EXTOBJID, s.FILE_NAME, s.INSERT_DATE  
				   FROM SPACEMGFILES s  INNER JOIN (SELECT FILE_NAME, MAX(INSERT_DATE) as MaxDate 
				   FROM SPACEMGFILES GROUP BY FILE_NAME) sm on s.FILE_NAME = sm.FILE_NAME AND s.INSERT_DATE = sm.MaxDate 
				   AND s.NODE_NAME='$cmd_args{nodename}' AND s.FILESPACE_NAME='$cmd_args{filespace}' AND s.FILE_NAME LIKE '$cmd_args{filename}%'\"";

    open (my $cmd_input_fh, "$command|") or die "Could not execute command!";
    open (my $FILE_LIST_OUT_FH, ">", "$cmd_args{filelist}") or die "Could not open file $!";
    
    my $file_count = 0;
    my $is_in_data_block = 0;
    my $capture_output_header = 1;

    my $filespace_name;
    my $file_name;
    my $ext_obj_id;
    my $header_text;

    while (<$cmd_input_fh>)
    { 
      	if ($_ =~ m/^\s*NODE_NAME:.+/)
    	{
    		$is_in_data_block = 1;
    		$capture_output_header = 0;
    	}    	

    	if ($_ =~ m/^\s*FILESPACE_NAME:(.+)/)
    	{
    		my $value = trim($1);
    		$filespace_name = "$value";
    	}

    	if ($_ =~ m/^\s*FILE_NAME:(.+)/)
    	{
    		my $value = trim($1);
    		$file_name = "$value";
    	}  

    	if ($_ =~ m/^\s*EXTOBJID:(.+)/)
    	{
    		my $value = trim($1);
    		$ext_obj_id = "$value";    		
    	}
    	if ($_ =~ m/^\s*INSERT_DATE:.+/)
    	{
    		$is_in_data_block = 0;
    		$file_count++;
    		print $FILE_LIST_OUT_FH "$ext_obj_id \"$filespace_name$file_name\"\n";    		
    	} 
    	if ($capture_output_header)
    	{
    		$header_text .= $_;
    	}   	    	
    }
    	
    close $cmd_input_fh;    
	close $FILE_LIST_OUT_FH;

	if ($? != 0)
	{
		print "$header_text\n";
		print "Filelist creation failed.\n\n";
	}
	else
	{
		print "\nFilelist creation successful.\n";
		print "Files found: $file_count\n";
		print "Filelist: " . abs_path($cmd_args{filelist}) . "\n\n";
	}	
}

sub trim 
{
	my $s = shift; 
	chomp($s);
	$s =~ s/^\s+|\s+$//g; 
	return $s
}

check_cmd_args(\%cmd_args);
create_file_list(\%cmd_args);
